<?php

namespace MailOptin\CampaignMonitorConnect;

use MailOptin\Core\Connections\ConnectionInterface;
use MailOptin\Core\EmailCampaign\CampaignLogRepository;
use MailOptin\Core\Repositories\EmailCampaignRepository;

class Connect extends AbstractCampaignMonitorConnect implements ConnectionInterface
{
    /**
     * @var string key of connection service. its important all connection name ends with "Connect"
     */
    public static $connectionName = 'CampaignMonitorConnect';

    public function __construct()
    {
        ConnectSettingsPage::get_instance();

        add_filter('mailoptin_registered_connections', array($this, 'register_connection'));

        add_action('init', [$this, 'campaign_log_public_preview']);

        parent::__construct();
    }

    public function campaign_log_public_preview()
    {
        if (isset($_GET['campaignmonitor_preview_type'], $_GET['uuid'])) {
            $preview_type = sanitize_text_field($_GET['campaignmonitor_preview_type']);

            if (!in_array($preview_type, ['text', 'html'])) {
                return;
            }

            $campaign_uuid = sanitize_text_field($_GET['uuid']);

            $campaign_log_id = absint($this->uuid_to_campaignlog_id($campaign_uuid));

            $type_method = 'retrieveContent' . ucfirst($preview_type);

            echo $this->replace_placeholder_tags(CampaignLogRepository::instance()->$type_method($campaign_log_id), $preview_type);
            exit;
        }
    }

    /**
     * Register Campaign Monitor Connection.
     *
     * @param array $connections
     *
     * @return array
     */
    public function register_connection($connections)
    {
        $connections[self::$connectionName] = __('Campaign Monitor', 'mailoptin');

        return $connections;
    }

    /**
     * Replace view webversion link with that of campaign monitor html tag.
     *
     * @return string
     */
    public function campaign_monitor_webversion_tag()
    {
        return '<webversion>[mo_header_web_version_link_label]</webversion>';
    }

    /**
     * Replace unsubscribe link with that of campaign monitor html tag.
     *
     * @return string
     */
    public function campaign_monitor_unsubscribe_tag()
    {
        return '<unsubscribe>[mo_footer_unsubscribe_link_label]</unsubscribe>';
    }

    /**
     * Fulfill interface contract.
     *
     * {@inheritdoc}
     */
    public function replace_placeholder_tags($content, $type = 'html')
    {
        if ($type == 'text') {
            $search = [
                '{{webversion}}',
                '{{unsubscribe}}'
            ];

            $replace = [
                '[webversion]',
                '[unsubscribe]',
            ];

            $content = str_replace($search, $replace, $content);
        }

        if ($type == 'html') {
            // use regex to replace the "view web version" and unsubribe mailoptin tag with campaignmonitor html tag equivalent.
            // see https://help.campaignmonitor.com/topic.aspx?t=97
            $pattern = ['/<a .+ href="{{webversion}}(?:.+)?">(.+)<\/a>/', '/<a .+ href="{{unsubscribe}}(?:.+)?">(.+)<\/a>/'];
            $replacement = ['<webversion>$1</webversion>', '<unsubscribe>$1</unsubscribe>'];
            $content = preg_replace($pattern, $replacement, $content);
        }

        // search and replace this if this oeration is for text content.
        return $this->replace_footer_placeholder_tags($content);
    }

    /**
     * {@inherit_doc}
     *
     * Return array of email list
     *
     * @return mixed
     */
    public function get_email_list()
    {
        try {
            $response = $this->campaignmonitorInstance()->getEmailList($this->client_id);

            // an array with list id as key and name as value.
            $lists_array = array();
            if (is_array($response) && !empty($response)) {
                foreach ($response as $list) {
                    $lists_array[$list->ListID] = $list->Name;
                }
            }

            return $lists_array;


        } catch (\Exception $e) {
            self::save_optin_error_log($e->getMessage());
        }
    }

    /**
     * @param int $email_campaign_id
     * @param int $campaign_log_id
     * @param string $subject
     * @param string $content_html
     * @param string $content_text
     *
     * @throws \Exception
     *
     * @return array
     */
    public function send_newsletter($email_campaign_id, $campaign_log_id, $subject, $content_html, $content_text)
    {
        return (new SendCampaign($email_campaign_id, $campaign_log_id, $subject, $content_html, $content_text))->send();
    }

    /**
     * @param string $email
     * @param string $name
     * @param string $list_id ID of email list to add subscriber to
     * @param mixed|null $extras
     *
     * @return mixed
     */
    public function subscribe($email, $name, $list_id, $extras = null)
    {
        return (new Subscription($email, $name, $list_id))->subscribe();
    }

    /**
     * Singleton poop.
     *
     * @return Connect|null
     */
    public static function get_instance()
    {
        static $instance = null;

        if (is_null($instance)) {
            $instance = new self();
        }

        return $instance;
    }
}