<?php

namespace MailOptin\MailChimpConnect;

use MailOptin\Core\Admin\Customizer\OptinForm\Customizer;
use MailOptin\Core\Connections\ConnectionInterface;
use MailOptin\Core\Repositories\EmailCampaignRepository;

class Connect extends AbstractMailChimpConnect implements ConnectionInterface
{
    /**
     * @var string key of connection service. its important all connection name ends with "Connect"
     */
    public static $connectionName = 'MailChimpConnect';

    public function __construct()
    {
        ConnectSettingsPage::get_instance();

        add_filter('mailoptin_registered_connections', array($this, 'register_connection'));

        add_filter('mo_optin_form_customizer_integration_settings', array($this, 'integration_customizer_settings'));
        add_filter('mo_optin_form_customizer_integration_controls', array($this, 'integration_customizer_controls'), 10, 4);

        // include mailchimp reward badge so their automatic footer won't be added for forever-free accounts.
        add_action('mailoptin_email_template_before_forge', function ($email_campaign_id) {
            $connect_service = EmailCampaignRepository::get_customizer_value($email_campaign_id, 'connection_service');

            if ($connect_service == 'MailChimpConnect') {
                add_filter('mailoptin_email_template_footer_description', [$this, 'add_reward_merge_tag']);
            } else {
                remove_filter('mailoptin_email_template_footer_description', [$this, 'add_reward_merge_tag']);
            }
        });

        parent::__construct();
    }

    /**
     * Register MailChimp Connection.
     *
     * @param array $connections
     *
     * @return array
     */
    public function register_connection($connections)
    {
        $connections[self::$connectionName] = __('MailChimp', 'mailoptin');

        return $connections;
    }

    /**
     * Replace placeholder tags with actual MailChimp tags.
     *
     * {@inheritdoc}
     */
    public function replace_placeholder_tags($content, $type = 'html')
    {
        $search = [
            '{{webversion}}',
            '{{unsubscribe}}'
        ];

        $replace = [
            '*|ARCHIVE|*',
            '*|UNSUB|*'
        ];

        $content = str_replace($search, $replace, $content);

        return $this->replace_footer_placeholder_tags($content);
    }

    /**
     * {@inherit_doc}
     *
     * Return array of email list
     *
     * @return mixed
     */
    public function get_email_list()
    {
        try {
            $response = $this->mc_list_instance()->getLists();

            // an array with list id as key and name as value.
            $lists_array = array();
            if (isset($response->lists) && is_array($response->lists)) {
                foreach ($response->lists as $list) {
                    $lists_array[$list->id] = $list->name;
                }
            }

            return $lists_array;


        } catch (\Exception $e) {
            self::save_optin_error_log($e->getMessage());
        }
    }

    /**
     * @param int $email_campaign_id
     * @param int $campaign_log_id
     * @param string $subject
     * @param string $content_html
     * @param string $content_text
     *
     * @throws \Exception
     *
     * @return array
     */
    public function send_newsletter($email_campaign_id, $campaign_log_id, $subject, $content_html, $content_text)
    {
        return (new SendCampaign($email_campaign_id, $campaign_log_id, $subject, $content_html, $content_text))->send();
    }

    /**
     * @param string $email
     * @param string $name
     * @param string $list_id ID of email list to add subscriber to
     * @param mixed|null $extras
     *
     * @return mixed
     */
    public function subscribe($email, $name, $list_id, $extras = null)
    {
        return (new Subscription($email, $name, $list_id, $extras))->subscribe();
    }

    /**
     * Forever free account requires the reward badge to be added in email.
     *
     * @param string $footer_description
     *
     * @return string
     */
    public function add_reward_merge_tag($footer_description)
    {
        // do not add MailChimp reward merge tag to customizer preview.
        if (!is_customize_preview()) {
            $footer_description .= '<br/>' . '*|IF:REWARDS|* *|HTML:REWARDS|* *|END:IF|*';
        }

        return $footer_description;
    }

    /**
     * @param array $settings
     *
     * @return mixed
     */
    public function integration_customizer_settings($settings)
    {
        $settings['MailChimpConnect_disable_double_optin'] = array(
            'default' => apply_filters('mailoptin_customizer_email_campaign_MailChimpConnect_disable_double_optin', false),
            'type' => 'option',
            'transport' => 'postMessage',
        );

        return $settings;
    }

    /**
     * @param array $controls
     * @param \WP_Customize_Manager $wp_customize
     * @param string $option_prefix
     * @param Customizer $customizerClassInstance
     *
     * @return mixed
     */
    public function integration_customizer_controls($controls, $wp_customize, $option_prefix, $customizerClassInstance)
    {
        // always prefix with the name of the connect/connection service.
        $controls['MailChimpConnect_disable_double_optin'] = apply_filters('mailoptin_customizer_settings_campaign_MailChimpConnect_disable_double_optin',
            array(
                'type' => 'checkbox',
                'label' => __('Disable Double Optin', 'mailoptin'),
                'section' => $customizerClassInstance->integration_section_id,
                'settings' => $option_prefix . '[MailChimpConnect_disable_double_optin]',
                'description' => __("Double optin requires users to confirm their email address before being added to a list (recommended).", 'mailoptin'),
                'priority' => 199
            )
        );

        return $controls;
    }

    /**
     * Singleton poop.
     *
     * @return Connect|null
     */
    public static function get_instance()
    {
        static $instance = null;

        if (is_null($instance)) {
            $instance = new self();
        }

        return $instance;
    }
}